package de.intevation.flys.artifacts.model;

import de.intevation.artifactdatabase.state.Facet;

import de.intevation.artifacts.Artifact;
import de.intevation.artifacts.CallContext;

import de.intevation.flys.artifacts.FLYSArtifact;

import de.intevation.flys.artifacts.access.RiverAccess;
import de.intevation.flys.artifacts.context.FLYSContext;

import de.intevation.flys.artifacts.math.MovingAverage;
import de.intevation.flys.artifacts.states.DefaultState.ComputeType;

import org.apache.log4j.Logger;

/**
 * Facet of a FlowVelocity curve.
 */
public class FlowVelocityFacet extends DataFacet {

    private static Logger logger = Logger.getLogger(FlowVelocityFacet.class);

    public FlowVelocityFacet() {
        // required for clone operation deepCopy()
    }


    public FlowVelocityFacet(
        int         idx,
        String      name,
        String      description,
        ComputeType type,
        String      stateId,
        String      hash
    ) {
        super(idx, name, description, type, hash, stateId);
    }


    public Object getData(Artifact artifact, CallContext context) {
        logger.debug("Get data for flow velocity at index: " + index);

        Double start = (Double)context.getContextValue("startkm");
        Double end = (Double)context.getContextValue("endkm");
        FLYSArtifact flys = (FLYSArtifact) artifact;

        CalculationResult res = (CalculationResult)
            flys.compute(context, hash, stateId, type, false);

        FlowVelocityData[] data = (FlowVelocityData[]) res.getData();
        if(start != null && end != null) {
            FLYSContext fc = (FLYSContext)context.globalContext();
            ZoomScale scales = (ZoomScale)fc.get("zoomscale");
            RiverAccess access = new RiverAccess((FLYSArtifact)artifact);
            String river = access.getRiver();

            double radius = scales.getRadius(river, start, end);
            FlowVelocityData oldData = data[index];
            FlowVelocityData newData = new FlowVelocityData();
            double[][] q = oldData.getQPoints();
            double[][] totalV = MovingAverage.weighted(oldData.getTotalChannelPoints(), radius);
            double[][] mainV = MovingAverage.weighted(oldData.getMainChannelPoints(), radius);
            double[][] tau = MovingAverage.weighted(oldData.getTauPoints(), radius);
            for(int j = 0; j < q[0].length; j++) {
                newData.addKM(q[0][j]);
                newData.addQ(q[1][j]);
                newData.addTauMain(tau[1][j]);
                newData.addVMain(mainV[1][j]);
                newData.addVTotal(totalV[1][j]);
            }
            return newData;
        }
        return data[index];
    }


    /** Copy deeply. */
    @Override
    public Facet deepCopy() {
        FlowVelocityFacet copy = new FlowVelocityFacet();
        copy.set(this);
        copy.type    = type;
        copy.hash    = hash;
        copy.stateId = stateId;
        return copy;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
